use goblin::mach::{cputype::get_arch_from_flag, Mach, MultiArch};

use crate::error::Error;

/// Mach-O fat binary reader
#[derive(Debug)]
pub struct FatReader<'a> {
    buffer: &'a [u8],
    fat: MultiArch<'a>,
}

impl<'a> FatReader<'a> {
    /// Parse a Mach-O FAT binary from a buffer
    pub fn new(buffer: &'a [u8]) -> Result<Self, Error> {
        match Mach::parse(buffer)? {
            Mach::Fat(fat) => Ok(Self { buffer, fat }),
            Mach::Binary(_) => Err(Error::NotFatBinary),
        }
    }

    /// Extract thin binary by arch name
    pub fn extract(&self, arch_name: &str) -> Option<&'a [u8]> {
        if let Some((cpu_type, _cpu_subtype)) = get_arch_from_flag(arch_name) {
            return self
                .fat
                .find_cputype(cpu_type)
                .unwrap_or_default()
                .map(|arch| arch.slice(self.buffer));
        }
        None
    }
}

impl<'a> std::ops::Deref for FatReader<'a> {
    type Target = MultiArch<'a>;

    fn deref(&self) -> &Self::Target {
        &self.fat
    }
}

impl<'a> std::ops::DerefMut for FatReader<'a> {
    fn deref_mut(&mut self) -> &mut Self::Target {
        &mut self.fat
    }
}
